from untei.constants import Const
from untei import utils
import sys
import os
from distutils import dir_util
import shutil
from untei import article
from untei import interfaces
theme_path = Const.FILES_PATH + 'theme'
sys.path.append(theme_path)


def read_config(path):
    config = {}

    config_lines = open(path).readlines()
    for l in config_lines:
        if not(l.startswith("#")):
            key, value = l.split(" = ")
            if is_config_key(key):
                config[key] = value
            else:
                print("Warn: Unknown key '" + key + "'")
        else:
            pass
    return config

def is_config_key(key):
    return key in Const.SITE_CONFIG_KEYS


class SiteConfig:
    def __init__(self, path):
        self.config = read_config(path)
        self.tags_to_article = {}
        self.authors_to_article = {}
        self.output_path = Const.OUTPUT_DIRECTORY
        self.template = open(Const.TEMPLATE_ARTICLE).read()
        self.articles_ordered_by_date = []

    def all_articles(self):
        return self.articles_ordered_by_date

    def get(self, key):
        return self.config[key]

    def add_articles(self, files):
        articles = []
        for f in files:
            if utils.file_type(f) == Const.FILE_TYPE_MARKDOWN:
                try:
                    a = article.Article(f)
                except:
                    print("Parse Error: " + f)
                articles.append(a)
                self.__update_tags_by_article__(a)
            else:
                pass

        self.articles_ordered_by_date = utils.order_articles_by_date(articles)

    def __update_tags_by_article__(self, a):
        for tag in a.tags:
            if tag in self.tags_to_article.keys():
                self.tags_to_article[tag].append(a)
            else:
                self.tags_to_article[tag] = [ a ]

    def __provide_interface_for_tag__(self, tag):
        variables = {
            "articles_tagged_as": self.tags_to_article[tag],
            "current_tag"       : tag
        }
        return variables

    def create_tag_pages(self):
        for tag in self.tags_to_article.keys():
            src = open(Const.SITE_CONFIG_TAG).read()

            interface = interfaces.tag_interface(self, tag)

            exec(src, interface, interface)
            var_value_mapping = {
                Const.TEMPLATE_CONTENT_INDICATOR: interface["body"],
                Const.TEMPLATE_SITE_TITLE_INDICATOR: self.config[Const.SITE_CONFIG_KEY_TITLE],
                Const.TEMPLATE_COPYRIGHT_INDICATOR: self.config[Const.SITE_CONFIG_KEY_COPYRIGHT],
                Const.TEMPLATE_ARTICLE_TITLE : "Tag: " + tag + " - " + self.config["title"]
            }
            content = utils.safe_apply_template(self.template , var_value_mapping)
            path = self.output_path + "/" + tag + ".html"
            with open(path, "w") as f:
                f.write(content)

    def copy_theme_files(self):
        files = os.listdir(theme_path)
        for f_name in files:
            f = theme_path + "/" + f_name

            if not(utils.is_uncopied(f_name)):
                if os.path.isdir(f):
                    dir_util.copy_tree(f, Const.OUTPUT_DIRECTORY + f_name)
                else:
                    shutil.copy2(f, Const.OUTPUT_DIRECTORY + f_name)

    def create_index_page(self):
        src = open(Const.SITE_CONFIG_INDEX).read()

        interface = interfaces.index_interface(self)

        exec(src, interface, interface)
        var_value_mapping = {
            Const.TEMPLATE_CONTENT_INDICATOR: interface["body"],
            Const.TEMPLATE_SITE_TITLE_INDICATOR: self.config[Const.SITE_CONFIG_KEY_TITLE],
            Const.TEMPLATE_COPYRIGHT_INDICATOR: self.config[Const.SITE_CONFIG_KEY_COPYRIGHT],
            Const.TEMPLATE_ARTICLE_TITLE : self.config["title"]
        }
        content = utils.safe_apply_template(self.template , var_value_mapping)

        path = self.output_path + "/index.html"

        with open(path, "w") as f:
            f.write(content)



import sys

def _path_concatinate(p1, p2):
    if p1[len(p1)-1] == "/" and p2[0] == "/":
        return p1 + p2[1:]
    elif p1[len(p1)-1] == "/" and p2[0] != "/":
        return p1 + p2
    elif p1[len(p1)-1] != "/" and p2[0] == "/":
        return p1 + p2
    else:
        return p1 + "/" + p2

class Const:
    # arguments
    FILES_PATH = sys.argv[1] + "/"
    # Essential file names
    OUTPUT_DIRECTORY =  _path_concatinate(FILES_PATH, "output/")
    SITE_CONFIG_PATH  = _path_concatinate(FILES_PATH, "theme/site_config.txt")
    SITE_CONFIG_KEY_TITLE = "title"
    SITE_CONFIG_KEY_COPYRIGHT = "copyright"
    SITE_CONFIG_KEYS = { SITE_CONFIG_KEY_TITLE, SITE_CONFIG_KEY_COPYRIGHT }
    SITE_CONFIG_TAG = _path_concatinate(FILES_PATH, "theme/tag.py")
    SITE_CONFIG_INDEX = _path_concatinate(FILES_PATH, "theme/index.py")
    SITE_CONFIG_AUTHOR = "/theme/author.py"

    # Article attributes
    ## Status
    ARTICLE_STATUS = "ArticleStatus"
    ARTICLE_STATUS_READY = "ArticleStatusReady"
    ARTICLE_STATUS_DRAFT = "ArticleStatusDraft"

    ## Title
    ARTICLE_TITLE = "ArticleTitle"

    ## Tags
    ARTICLE_TAG = "ArticleTag"
    ARTICLE_TAGS = "ArticleTags"

    ## Authors
    ARTICLE_AUTHOR = "ArticleAuthor"
    ARTICLE_AUTHORS = "ArticleAuthors"

    ## Template
    TEMPLATE_ARTICLE = FILES_PATH + "theme/template.html"
    TEMPLATE_SITE_TITLE_INDICATOR = "{{site_config.title}}"
    TEMPLATE_COPYRIGHT_INDICATOR = "{{site_config.copyright}}"
    TEMPLATE_CONTENT_INDICATOR = "{{article.content}}"
    TEMPLATE_ARTICLE_TITLE = "{{article.title}}"
    TEMPLATE_VARIABLES = [ TEMPLATE_SITE_TITLE_INDICATOR, TEMPLATE_CONTENT_INDICATOR, TEMPLATE_COPYRIGHT_INDICATOR ]

    # File type
    FILE_TYPE = "FileType"
    FILE_TYPE_MARKDOWN = "FileTypeMarkdown"
    FILE_TYPE_DIRECTORY = "FileTypeDirectory"
    EXTENSIONS_FOR_MARKDOWN = [ ".md", ".markdown", ".mdown" ]
    FILE_TYPE_HTML = "FileTypeHtml"
    EXTENSIONS_FOR_HTML = [ ".html", ".htm" ]




    # other keywords
    FILE_PATH = "FilePath"



def index_interface(config):
    def latest_articles(n):
        return __latest_articles__(config, None, n)

    def latest_articles_tagged_as(tag, n):
        return __latest_articles__(config, tag, n)


    variables = {
        "latest_articles"          : latest_articles,
        "latest_articles_tagged_as": latest_articles_tagged_as,
        "all_tags"                 : config.tags_to_article.keys(),
        "body"                     : ""
    }

    return variables

def tag_interface(config, tag):
    def latest_tagged_articles(n):
        return __latest_articles__(config, tag, n)
    variables = {
        "latest_tagged_articles": latest_tagged_articles,
        "tag"                   : tag,
        "tagged_articles"       : config.tags_to_article[tag],
        "body"                  : ""
    }

    return variables

def __latest_articles__(config, tag, n):
    if tag == None:
        return config.articles_ordered_by_date[:n]
    articles_list = []
    for a in config.articles_ordered_by_date:
        if tag in a.tags:
            articles_list.append(a)

    return articles_list[:n]



from distutils.core import setup

entry_points = {
    'console_scripts': [
        'untei = untei.static_site_generator:main'
    ]
}

dependencies = ['parsimonious']
setup(
    name = "untei",
    packages = ["untei"],
    version = "0.0.12",
    description = "Static site generator you can create your own theme by python coding",
    author = "Hara Yuki",
    author_email = "youhui.dev@gmail.com",
    url = "",
    download_url = "",
    keywords = ["static site generator"],
    entry_points = entry_points,
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Text Processing :: Markup",
        ],
    install_requires = dependencies,
    long_description = """\
Untei is a static site generator. You can configure blog-like websites with markdown documents.
The distictive point of untei from other site generator is that you can create your own theme as if you code in python.
"""
)



import os.path
import random
import re
from untei.constants import Const

def user_input_yes_or_no(ans):
    if ans.lower() in [ "yes", "y", "yeah", "yep", "yup" ]:
        return True
    elif ans.lower() in [ "no", "n", "nope" ]:
        return False
    else:
        return None

def is_uncopied(filename):
    if filename in ["site_config.txt", "template.html", "__pycache__"] :
        return True
    elif re.fullmatch(".*[.]pyc{0,1}", filename):
        return True
    else:
        return False

def safe_apply_template(template, assignment_dict):
    safe_assignment_dict = {}
    for check_key in assignment_dict.keys():
        for key in assignment_dict.keys():
            if key in assignment_dict[check_key]:
                # create a temporary key
                while True:
                    temp_key = str(random.random())
                    if not(temp_key in assignment_dict[key]) and not(temp_key in template):
                        break

                # Evacuation
                safe_assignment_dict[temp_key] = key
                assignment_dict[check_key] = assignment_dict[check_key].replace(key, temp_key)
            else:
                pass
    rep = replace_with_dict(template, assignment_dict)
    rep = replace_with_dict(rep, safe_assignment_dict)
    return rep


def replace_with_dict(target, assignment_dict):
    rep = target
    for key in assignment_dict.keys():
        rep = rep.replace(key, assignment_dict[key])
    return rep


def order_articles_by_date(articles):
    n = len(articles)
    if n <= 1:
        return articles

    pivot_date = articles[int(0.5 * n)].date

    later   = []
    earlier = []

    for a in articles:
        if a.date >= pivot_date:
            later.append(a)
        else:
            earlier.append(a)

    if len(later) <= 1 or len(earlier) <= 1:
        later.extend(earlier)
        return later
    else:
        earlier = order_articles_by_date(earlier)
        later   = order_articles_by_date(later)
        later.extend(earlier)
        return later

def file_type(file_name):
    root, ext = os.path.splitext(file_name)
    if ext in Const.EXTENSIONS_FOR_MARKDOWN:
        return Const.FILE_TYPE_MARKDOWN
    elif ext in Const.EXTENSIONS_FOR_HTML:
        return Const.FILE_TYPE_HTML
    elif ext == "":
        return Const.FILE_TYPE_DIRECTORY
    else:
        print("Warn: Find an extension out of scope: ", ext)
        return ext

def file_name(path):
    return os.path.basename(path).split(".")[0]


def does_include_template_variables(s):
    rtn = False
    for var in Const.TEMPLATE_VARIABLES:
        rtn = rtn or (var in s)

    return rtn

def handled_files():
    return [f for f in os.listdir(Const.FILES_PATH) if file_type(f) != Const.FILE_TYPE_DIRECTORY]
